UserDefinedOperator

Enum UserDefinedOperator 

pub enum UserDefinedOperator {
Show 16 variants BitwiseAnd, BitwiseNot, Negate, BitwiseOr, BitwiseXor, Add, Divide, Modulo, Multiply, Subtract, Equal, More, MoreEqual, Less, LessEqual, NotEqual,
}
A user-defined operator.

See also the Solidity blog post on user-defined operators.

Variants



BitwiseAnd

&



BitwiseNot

~



Negate

-

Note that this is the same as Subtract, and that it is currently not being parsed.



BitwiseOr

|



BitwiseXor

^



Add

+



Divide

/



Modulo

%



Multiply

*



Subtract

-



Equal

==



More

>



MoreEqual

>=



Less

<



LessEqual

<=



NotEqual

!=

Implementations

Source

impl UserDefinedOperator

pub const fn as_str(&self) -> &'static str

Returns the string representation of this type.

Source

impl UserDefinedOperator

pub const fn args(&self) -> usize

Returns the number of arguments needed for this operator’s operation.

pub const fn is_unary(&self) -> bool

Returns whether self is a unary operator.

pub const fn is_binary(&self) -> bool

Returns whether self is a binary operator.

pub const fn is_bitwise(&self) -> bool

Returns whether self is a bitwise operator.

pub const fn is_arithmetic(&self) -> bool

Returns whether self is an arithmetic operator.

pub const fn is_comparison(&self) -> bool

Returns whether this is a comparison operator.